<?php
/**
 * WP File Download
 *
 * @package WP File Download
 * @author  Joomunited
 * @version 1.0
 */

// no direct access
defined('ABSPATH') || die();

use Joomunited\WPFramework\v1_0_5\Application;

/**
 * Class WpfdAddonDropbox
 */
class WpfdAddonDropbox
{
    /**
     * Parameters
     *
     * @var array
     */
    protected $params;

    /**
     * Application name
     *
     * @var string
     */
    protected $appName = 'JoomUnited/1.0';
    /**
     * Last error
     *
     * @var string
     */
    protected $lastError;

    /**
     * Dropbox client
     *
     * @var Dropbox\Client
     */
    protected $client;

    /**
     * Drive type
     *
     * @var string
     */
    protected $type = 'dropbox';

    /**
     * WpfdAddonDropbox constructor.
     */
    public function __construct()
    {
        $app = Application::getInstance('Wpfd');
        $path_wpfdhelperfile = $app->getPath() . DIRECTORY_SEPARATOR . 'site' . DIRECTORY_SEPARATOR . 'helpers';
        $path_wpfdhelperfile .= DIRECTORY_SEPARATOR . 'WpfdHelperFile.php';
        require_once $path_wpfdhelperfile;
        set_include_path(__DIR__ . PATH_SEPARATOR . get_include_path());
        require_once 'Dropbox/autoload.php';
        $this->loadParams();
    }

    /**
     * Get last error
     *
     * @return mixed
     */
    public function getLastError()
    {
        return $this->lastError;
    }

    /**
     * Get data config by dropbox
     *
     * @param string $name Option name
     *
     * @return array|null
     */
    public function getDataConfigByDropbox($name)
    {
        return WpfdAddonHelper::getDataConfigByDropbox($name);
    }

    /**
     * Get all dropbox config
     *
     * @return array
     */
    public function getAllDropboxConfigs()
    {
        return WpfdAddonHelper::getAllDropboxConfigs();
    }

    /**
     * Save database
     *
     * @param array $data Config data
     *
     * @return boolean
     */
    public function saveDropboxConfigs($data)
    {
        return WpfdAddonHelper::saveDropboxConfigs($data);
    }

    /**
     * Load parameters
     *
     * @return void
     */
    protected function loadParams()
    {
        $params       = $this->getDataConfigByDropbox('dropbox');
        $this->params = new stdClass();

        $this->params->dropboxKey          = isset($params['dropboxKey']) ? $params['dropboxKey'] : '';
        $this->params->dropboxSecret       = isset($params['dropboxSecret']) ? $params['dropboxSecret'] : '';
        $this->params->dropboxToken        = isset($params['dropboxToken']) ? $params['dropboxToken'] : '';
        $this->params->dropboxBaseFolderId = isset($params['dropboxBaseFolderId']) ?
            $params['dropboxBaseFolderId'] : '';
    }

    /**
     * Save parameters
     *
     * @return void
     */
    protected function saveParams()
    {
        $params                       = $this->getAllDropboxConfigs();
        $params['dropboxKey']         = $this->params->dropboxKey;
        $params['dropboxSecret']      = $this->params->dropboxSecret;
        $params['dropboxToken']       = $this->params->dropboxToken;
        $params['dropboxConnectedBy'] = get_current_user_id();
        $this->saveDropboxConfigs($params);
    }

    /**
     * Get web authenticate
     *
     * @return \Dropbox\WebAuthNoRedirect
     */
    public function getWebAuth()
    {
        $dropboxKey = '';
        $dropboxSecret = 'dropboxSecret';

        if (!empty($this->params->dropboxKey)) {
            $dropboxKey = $this->params->dropboxKey;
        }
        if (!empty($this->params->dropboxSecret)) {
            $dropboxSecret = $this->params->dropboxSecret;
        }

        $appInfo = new Dropbox\AppInfo($dropboxKey, $dropboxSecret);
        $webAuth = new Dropbox\WebAuthNoRedirect($appInfo, $this->appName);

        return $webAuth;
    }

    /**
     * Get authorize Url allow user
     *
     * @return string
     */
    public function getAuthorizeDropboxUrl()
    {
        $authorizeUrl = $this->getWebAuth()->start();

        return $authorizeUrl;
    }

    /**
     * Convert the authorization code into an access token.
     *
     * @param string $authCode Authenticate code
     *
     * @return array
     */
    public function convertAuthorizationCode($authCode)
    {
        list($accessToken, $dropboxUserId) = $this->getWebAuth()->finish($authCode);
        $list = array('accessToken' => $accessToken, 'dropboxUserId' => $dropboxUserId);
        return $list;
    }

    /**
     * Check Dropbox Token
     *
     * @return boolean
     */
    public function checkAuth()
    {
        $dropboxToken = $this->params->dropboxToken;
        if (!empty($dropboxToken)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Log out dropbox
     *
     * @return void
     */
    public function logout()
    {
        $params                  = $this->getAllDropboxConfigs();
        $params['dropboxKey']    = $this->params->dropboxKey;
        $params['dropboxSecret'] = $this->params->dropboxSecret;
        $params['dropboxAuthor'] = '';
        $params['dropboxToken']  = '';
        $this->saveDropboxConfigs($params);
    }

    /**
     * Get Dropbox account
     *
     * @return \Dropbox\Client
     */
    public function getAccount()
    {
        if (!$this->client) {
            $dropboxToken = $this->params->dropboxToken;
            $this->client = new Dropbox\Client($dropboxToken, $this->appName);
        }

        return $this->client;
    }

    /**
     * Create Folder to Dropbox
     *
     * @param string  $title  Title
     * @param integer $parent Category parent id
     *
     * @return array|boolean|null
     */
    public function createDropFolder($title, $parent)
    {
        try {
            $dropbox = $this->getAccount();
            $pathFolder = '';
            if ($parent === 0) {
                $baseFolderId = $this->params->dropboxBaseFolderId;

                if (!empty($baseFolderId)) {
                    $baseFolderMeta = $dropbox->getMetadata($baseFolderId);
                    $pathFolder = $baseFolderMeta['path_lower'];
                }
            } else {
                $baseFolderId = WpfdAddonHelper::getDropboxIdByTermId($parent);
                $pathFolder   = WpfdAddonHelper::getPathByDropboxId($baseFolderId);
            }

            $path = $pathFolder . '/' . $title;

            $result = $dropbox->createFolder($path);
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }

        return $result;
    }

    /**
     * Delete Folder to dropbox
     *
     * @param string $id Folder id
     *
     * @return boolean
     */
    public function deleteDropbox($id)
    {
        try {
            $dropbox = $this->getAccount();
            $dropbox->delete($id);
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();
            return false;
        }

        return true;
    }

    /**
     * Upload file to Folder Dropbox
     *
     * @param string  $filename  File name
     * @param string  $fileTemp  Temp file path
     * @param integer $size      File size
     * @param string  $id_folder Folder id
     *
     * @return boolean|mixed
     */
    public function uploadFile($filename, $fileTemp, $size, $id_folder)
    {
        $f    = fopen($fileTemp, 'rb');
        $path = $id_folder . '/' . $filename;
        try {
            $dropbox = $this->getAccount();
            $result  = $dropbox->uploadFile($path, Dropbox\WriteMode::add(), $f, $size);
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }

        return $result;
    }

    /**
     * Rename Folder Dropbox
     *
     * @param string $id       File id
     * @param string $filename File name
     *
     * @return boolean
     */
    public function changeDropboxFilename($id, $filename)
    {
        try {
            $dropbox = $this->getAccount();
            return $dropbox->move($id, $filename);
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();
            return false;
        }

        return true;
    }

    /**
     * Get all files in dropbox
     *
     * @param string $idFolder Folder id
     *
     * @return mixed
     */
    public function getAllFiles($idFolder)
    {
        $dropbox = $this->getAccount();
        $fs      = $dropbox->getMetadataWithChildren($idFolder);

        return $fs['entries'];
    }

    /**
     * Download thumbnail for file
     *
     * @param string $idFile File id
     * @param string $path   Save path
     *
     * @return boolean
     */
    public function downloadThumbnail($idFile, $path)
    {
        $dropbox = $this->getAccount();
        $fd = fopen($path, 'wb');
        try {
            $dropbox->getThumbnailV2($idFile, $fd, 'png', 'w1024h768');
            if (file_exists($path)) {
                return $path;
            }
            return false;
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();
            fclose($fd);
            unlink($path);
            return false;
        }
        return false;
    }
    /**
     * Sub val Sort
     *
     * @param array  $a         Array to sort
     * @param string $subkey    Key to sort
     * @param string $direction Sort direction
     *
     * @return array
     */
    private function subvalSort($a, $subkey, $direction)
    {
        if (empty($a)) {
            return $a;
        }
        foreach ($a as $k => $v) {
            $b[$k] = strtolower($v->$subkey);
        }
        if (strtolower($direction) === 'asc') {
            asort($b);
        } else {
            arsort($b);
        }
        $c = array();
        foreach ($b as $key => $val) {
            $c[] = $a[$key];
        }

        return $c;
    }

    /**
     * List dropbox files
     *
     * @param string  $folder_id     Folder id
     * @param integer $idCategory    Category id
     * @param string  $ordering      Ordering
     * @param string  $direction     Ordering direction
     * @param boolean $list_id_files List id files
     *
     * @return array|boolean
     */
    public function listDropboxFiles(
        $folder_id,
        $idCategory,
        $ordering = 'ordering',
        $direction = 'asc',
        $list_id_files = false
    ) {
        try {
            $dropbox = $this->getAccount();
            $fs      = $dropbox->getMetadataWithChildren($folder_id);
            if (empty($fs)) {
                return false;
            }
            $files = array();
            //to fix special characters in file name like german "Umlaute"
            setlocale(LC_ALL, 'en_US.UTF-8');
            foreach ($fs['entries'] as $f) {
                if (is_array($list_id_files) && !in_array($f['id'], $list_id_files)) {
                    continue;
                }
                if ($f['.tag'] === 'file') {
                    $info  = pathinfo($f['path_display']);
                    $fsize = $f['size'];

                    //link download && linkview
                    $config = get_option('_wpfd_global_config');
                    if (!empty($config) && !empty($config['date_format'])) {
                        $dateFormat = $config['date_format'];
                    } else {
                        $dateFormat = 'd-m-Y h:i:s';
                    }
                    if (empty($config) || empty($config['uri'])) {
                        $seo_uri = 'download';
                    } else {
                        $seo_uri = rawurlencode($config['uri']);
                    }
                    if (empty($config) || !isset($config['rmdownloadext'])) {
                        $rmdownloadext = false;
                    } else {
                        $rmdownloadext = ((int) $config['rmdownloadext'] === 1) ? true : false;
                    }
                    $perlink       = get_option('permalink_structure');
                    $rewrite_rules = get_option('rewrite_rules');
                    $idFolderTitle = substr($folder_id, 1);

                    if (!empty($rewrite_rules)) {
                        if (strrpos($idFolderTitle, '/') !== false) {
                            $idFolderTitle = substr($idFolderTitle, strrpos($idFolderTitle, '/') + 1);
                        }
                        if (strpos($perlink, 'index.php')) {
                            $linkdownload = get_site_url() . '/index.php/' . $seo_uri . '/' . $idCategory . '/';
                            $linkdownload .= $idFolderTitle . '/' . $f['id'] . '/' . $info['filename'];
                        } else {
                            $linkdownload = get_site_url() . '/' . $seo_uri . '/' . $idCategory . '/';
                            $linkdownload .= $idFolderTitle . '/' . $f['id'] . '/' . $info['filename'];
                        }
                        $linkdownload = $linkdownload . (!$rmdownloadext ? '.' . $info['extension'] : '');
                    } else {
                        $linkdownload = admin_url('admin-ajax.php') . '?juwpfisadmin=false&action=wpfd&task';
                        $linkdownload .= '=file.download&wpfd_category_id=' . $idCategory . '&wpfd_file_id=' . $f['id'];
                    }

                    if (defined('WPFD_DROPBOX_DIRECT') && WPFD_DROPBOX_DIRECT) {
                        // Generate temporary download, this link live for 4 hour
                        $linkdownload = $dropbox->createTemporaryDirectLink($f['path_lower']);
                    }
                    $file             = new stdClass();
                    $file->id         = $f['id'];
                    $file->ID         = $file->id;
                    $file->title      = $info['filename'];
                    $file->post_title = $file->title;
                    $file->post_name  = $file->title;
                    if (!empty($info['extension'])) {
                        $file->ext = $info['extension'];
                    } else {
                        $file->ext = '';
                    }
                    $file->size = $fsize;
                    if (!empty($f['client_modified'])) {
                        $file->created_time = get_date_from_gmt(date('Y-m-d H:i:s', strtotime($f['client_modified'])));
                        $file->created      = mysql2date($dateFormat, $file->created_time);
                    } else {
                        $file->created_time = '';
                        $file->created = '';
                    }

                    $file->modified_time    = get_date_from_gmt(date('Y-m-d H:i:s', strtotime($f['server_modified'])));
                    $file->modified         = mysql2date($dateFormat, $file->modified_time);
                    $file->ordering         = 0;
                    $file->file_tags        = '';
                    $file->linkdownload     = $linkdownload;
                    $file->hits             = 0;
                    $file->version          = '';
                    $file->versionNumber    = '';
                    $file->description      = '';
                    $file->file_tags        = '';
                    $file->file_custom_icon = '';
                    $config                 = get_option('_wpfdAddon_dropbox_fileInfo');
                    if (!empty($config) && isset($config[$idCategory]) && isset($config[$idCategory][$f['id']])) {
                        $file->description   = isset($config[$idCategory][$f['id']]['description']) ?
                            $config[$idCategory][$f['id']]['description'] : '';
                        $file->version       = isset($config[$idCategory][$f['id']]['version']) ?
                            $config[$idCategory][$f['id']]['version'] : '';
                        $file->versionNumber = $file->version;
                        $file->hits          = isset($config[$idCategory][$f['id']]['hits']) ?
                            $config[$idCategory][$f['id']]['hits'] : 0;
                        $file->file_tags     = isset($config[$idCategory][$f['id']]['file_tags']) ?
                            $config[$idCategory][$f['id']]['file_tags'] : '';
                        $file->state = isset($config[$idCategory][$f['id']]['state']) ?
                            $config[$idCategory][$f['id']]['state'] : '1';
                        $file->canview = isset($config[$idCategory][$f['id']]['canview']) ?
                            $config[$idCategory][$f['id']]['canview'] : '';
                    }

                    $file->catid = $idCategory;
                    $files[]     = apply_filters('wpfda_file_info', $file, $this->type);
                    unset($file);
                }
            }
            $files = $this->subvalSort($files, $ordering, $direction);
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();
            return false;
        }

        return $files;
    }

    /**
     * Delete a file Dropbox
     *
     * @param string $id_file File id
     *
     * @return   boolean
     * @internal param $id
     * @internal param null $cloud_id
     */
    public function deleteFileDropbox($id_file)
    {
        try {
            $dropbox = $this->getAccount();
            $fs      = $dropbox->getMetadata($id_file);
            $dropbox->delete($fs['path_lower']);
            $dropboxUpload = get_option('_wpfdAddon_dropbox_file_user_upload') ? get_option('_wpfdAddon_dropbox_file_user_upload') : array();
            if (!empty($dropboxUpload)) {
                if (array_key_exists($id_file, $dropboxUpload)) {
                    unset($dropboxUpload[$id_file]);
                    update_option('_wpfdAddon_dropbox_file_user_upload', $dropboxUpload);
                }
            }
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }

        return true;
    }

    /**
     * Copy file dropbox
     *
     * @param string $path_from File path
     * @param string $path_to   Destination file path
     *
     * @return mixed|string
     */
    public function copyFileDropbox($path_from, $path_to)
    {
        try {
            $dropbox = $this->getAccount();
            return $dropbox->copy($path_from, $path_to);
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }
    }

    /**
     * Rename Folder Dropbox
     *
     * @param string $from From path
     * @param string $to   Destination path
     *
     * @return boolean|mixed
     */
    public function moveFile($from, $to)
    {
        try {
            $dropbox = $this->getAccount();

            return $dropbox->move($from, $to);
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }
    }

    /**
     * Get a file in dropbox
     *
     * @param string $idFile File id
     *
     * @return   array|boolean
     * @internal param $id
     * @internal param null $cloud_id
     */
    public function getDropboxFile($idFile)
    {
        try {
            $dropbox = $this->getAccount();
            $v       = $dropbox->getFileMetadata($idFile);
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }

        return $v;
    }

    /**
     * Get dropbox file info
     *
     * @param string  $idFile     File id
     * @param string  $idFolder   Folder name
     * @param integer $idCategory Term id
     * @param string  $token      Token key
     *
     * @return   array|boolean
     * @internal param $id
     * @internal param null $cloud_id
     */
    public function getDropboxFileInfos($idFile, $idFolder, $idCategory, $token = '')
    {
        try {
            $dropbox = $this->getAccount();
            $v       = $dropbox->getFileMetadata($idFile);
            //to fix special characters in file name like german "Umlaute"
            setlocale(LC_ALL, 'en_US.UTF-8');
            $info  = pathinfo($v['path_display']);
            $fsize = $v['size'];

            $config = get_option('_wpfd_global_config');
            if (!empty($config) && !empty($config['date_format'])) {
                $dateFormat = $config['date_format'];
            } else {
                $dateFormat = 'd-m-Y h:i:s';
            }
            if (empty($config) || empty($config['uri'])) {
                $seo_uri = 'download';
            } else {
                $seo_uri = rawurlencode($config['uri']);
            }
            $params           = $this->getGlobalConfig();
            $viewer_type      = WpfdBase::loadValue($params, 'use_google_viewer', 'lightbox');
            $extension_list   = 'png,jpg,pdf,ppt,pptx,doc,docx,xls,xlsx,dxf,ps,eps,xps,psd,tif,tiff,bmp,svg,pages,ai,dxf,ttf,txt,mp3,mp4';
            $extension_viewer = explode(',', WpfdBase::loadValue($params, 'extension_viewer', $extension_list));
            $extension_viewer = array_map('trim', $extension_viewer);
            $perlink          = get_option('permalink_structure');
            $rewrite_rules    = get_option('rewrite_rules');
            $idFolderTitle    = substr($idFolder, 1);
            if (empty($config) || !isset($config['rmdownloadext'])) {
                $rmdownloadext = false;
            } else {
                $rmdownloadext = ((int) $config['rmdownloadext'] === 1) ? true : false;
            }
            if (!empty($rewrite_rules)) {
                if (strrpos($idFolderTitle, '/') !== false) {
                    $idFolderTitle = substr($idFolderTitle, strrpos($idFolderTitle, '/') + 1);
                }
                if (strpos($perlink, 'index.php')) {
                    $linkdownload = get_site_url() . '/index.php/' . $seo_uri . '/' . $idCategory . '/';
                    $linkdownload .= $idFolderTitle . '/' . $idFile . '/' . $info['filename'];
                } else {
                    $linkdownload = get_site_url() . '/' . $seo_uri . '/' . $idCategory . '/' . $idFolderTitle;
                    $linkdownload .= '/' . $idFile . '/' . $info['filename'];
                    $linkdownload = $linkdownload . (!$rmdownloadext ? '.' . $info['extension'] : '');
                }
            } else {
                $linkdownload = admin_url('admin-ajax.php') . '?juwpfisadmin=false&action=wpfd&task=file.download';
                $linkdownload .= '&wpfd_category_id=' . $idCategory . '&wpfd_file_id=' . $idFile;
            }
            if (defined('WPFD_DROPBOX_DIRECT') && WPFD_DROPBOX_DIRECT) {
                // Generate temporary download, this link live for 4 hour
                $t = $dropbox->createTemporaryDirectLink($v['path_lower']);
                $linkdownload = $t;
            }
            $data                  = array();
            $data['linkdownload']  = $linkdownload;
            $data['ID']            = $v['id'];
            $data['id']            = $data['ID'];
            $data['state']         = isset($v['state']) ? (string) $v['state'] : '1';
            $data['catid']         = $idCategory;
            $data['title']         = $info['filename'];
            $data['post_title']    = $data['title'];
            $data['file']          = $info['basename'];
            $data['ext']           = $info['extension'];
            $data['created_time']  = get_date_from_gmt(date('Y-m-d H:i:s', strtotime($v['client_modified'])));
            $data['modified_time'] = get_date_from_gmt(date('Y-m-d H:i:s', strtotime($v['server_modified'])));
            $data['created']       = mysql2date($dateFormat, $data['created_time']);
            $data['modified']      = mysql2date($dateFormat, $data['modified_time']);
            $data['file_tags']     = '';
            $data['size']          = $fsize;
            $data['ordering']      = 1;
            $remote_url            = (isset($file_meta) && isset($file_meta['remote_url'])) ? $file_meta['remote_url'] : false;
            if ($viewer_type !== 'no' && in_array($info['extension'], $extension_viewer) && ($remote_url === false)) {
                $data['viewer_type'] = $viewer_type;
                $data['viewerlink']  = WpfdHelperFile::isMediaFile($info['extension']) ?
                    WpfdHelperFile::getMediaViewerUrl(
                        $v['id'],
                        $idCategory,
                        $info['extension']
                    ) : WpfdHelperFile::getViewerUrl($v['id'], $idCategory, $token);
            }
            $config = get_option('_wpfdAddon_dropbox_fileInfo');
            if (!empty($config) && isset($config[$idCategory]) && isset($config[$idCategory][$v['id']])) {
                $data['description']             = isset($config[$idCategory][$v['id']]['description']) ?
                    $config[$idCategory][$v['id']]['description'] : '';
                $data['version']                 = isset($config[$idCategory][$v['id']]['version']) ?
                    $config[$idCategory][$v['id']]['version'] : '';
                $data['versionNumber']           = $data['version'];
                $data['hits']                    = isset($config[$idCategory][$v['id']]['hits']) ?
                    $config[$idCategory][$v['id']]['hits'] : 0;
                $data['social']                  = isset($config[$idCategory][$v['id']]['social']) ?
                    $config[$idCategory][$v['id']]['social'] : 0;
                $data['file_tags']               = isset($config[$idCategory][$v['id']]['file_tags']) ?
                    $config[$idCategory][$v['id']]['file_tags'] : '';
                $data['file_multi_category']     = isset($config[$idCategory][$v['id']]['file_multi_category']) ?
                    $config[$idCategory][$v['id']]['file_multi_category'] : '';
                $data['file_multi_category_old'] = isset($config[$idCategory][$v['id']]['file_multi_category_old']) ?
                    $config[$idCategory][$v['id']]['file_multi_category_old'] : '';
                if (isset($config[$idCategory][$v['id']]['canview'])) {
                    $data['canview'] = $config[$idCategory][$v['id']]['canview'];
                }
            }

            $data = apply_filters('wpfda_file_info', $data, $this->type);
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }

        return $data;
    }

    /**
     * Change file title
     *
     * @param array $datas File info
     *
     * @return boolean
     */
    public function saveDropboxFileInfos($datas)
    {
        try {
            $dropbox = $this->getAccount();
            $getFile = $dropbox->getMetadata($datas['id']);
            $fpath   = pathinfo($getFile['path_lower']);
            $newpath = $fpath['dirname'] . '/' . $datas['title'] . '.' . $fpath['extension'];
            $dropbox->move($getFile['path_lower'], $newpath);
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }

        return true;
    }

    /**
     * Change version
     *
     * @param array $datas Version data
     *
     * @return boolean|mixed
     */
    public function saveDropboxVersion($datas)
    {
        try {
            $dropbox = $this->getAccount();
            $getFile = $dropbox->getMetadata($datas['old_file']);
            $f       = fopen($datas['new_tmp_name'], 'rb');
            $result  = $dropbox->updateFile(
                $getFile['path_lower'],
                $getFile['rev'],
                Dropbox\WriteMode::add(),
                $f,
                $datas['new_file_size']
            );
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }

        return $result;
    }

    /**
     * Get version info
     *
     * @param string  $idFile     File id
     * @param integer $idCategory Term id
     *
     * @return   array|boolean
     * @internal param $dropboxId
     */
    public function displayDropboxVersionInfo($idFile, $idCategory)
    {
        try {
            $dropbox  = $this->getAccount();
            $getFile  = $dropbox->getMetadata($idFile);
            $result   = $dropbox->listRevisions($getFile['path_lower'], 10);
            $versions = array();
            foreach ($result['entries'] as $v) {
                if ($getFile['rev'] !== $v['rev']) {
                    $fpath                   = pathinfo($v['path_lower']);
                    $version['ext']          = $fpath['extension'];
                    $version['size']         = $v['size'];
                    $version['catid']        = $idCategory;
                    $version['id']           = $v['id'];
                    $version['created_time'] = $v['client_modified'];
                    $version['meta_id']      = $v['rev'];
                    $versions[]              = $version;
                }
            }

            return $versions;
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }
    }

    /**
     * Restore version
     *
     * @param string $id_file File id
     * @param string $vid     Version id
     *
     * @return boolean
     */
    public function restoreVersion($id_file, $vid)
    {
        try {
            $dropbox = $this->getAccount();
            $getFile = $dropbox->getMetadata($id_file);
            $dropbox->restoreFile($getFile['path_lower'], $vid);

            return true;
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }
    }

    /**
     * Download version
     *
     * @param string $id_file File id
     * @param string $vid     Version id
     *
     * @return boolean
     */
    public function downloadVersion($id_file, $vid)
    {
        try {
            $dropbox  = $this->getAccount();
            $getFile  = $dropbox->getMetadata($id_file);
            $pinfo    = pathinfo($getFile['path_lower']);
            $tempfile = $pinfo['basename'];
            $fd       = fopen($tempfile, 'wb');
            $dropbox->getFile($getFile['path_lower'], $fd, $vid);
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename="' . basename($tempfile) . '"');
            header('Expires: 0');
            header('Cache-Control: must-revalidate');
            header('Pragma: public');
            header('Content-Length: ' . filesize($tempfile));
            readfile($tempfile);
            unlink($tempfile);
            exit;
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }
    }

    /**
     * Change dropbox order
     *
     * @param string $move     Move path
     * @param string $location New location path
     * @param string $parent   Parent
     *
     * @return boolean|mixed
     */
    public function changeDropboxOrder($move, $location, $parent)
    {
        try {
            if ($parent !== 0) {
                $fpath       = pathinfo($move);
                $baseMove    = '/' . $fpath['basename'];
                $newlocation = $location . $baseMove;
                $dropbox     = $this->getAccount();
                $result      = $dropbox->move($move, $newlocation);
            } else {
                $pinfo    = pathinfo($move);
                $basemove = '/' . $pinfo['basename'];
                $dropbox  = $this->getAccount();
                $result   = $dropbox->move($move, $basemove);
            }
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }

        return $result;
    }

    /**
     * Get list children folder
     *
     * @param string $baseFolderId Folder id
     *
     * @return array|boolean
     */
    public function listChildrenFolders($baseFolderId = '')
    {
        $dropbox    = $this->getAccount();
        $listfolder = array();
        try {
            $folderMetadatas = $dropbox->getMetadataWithChildren($baseFolderId, false);
            if (count($folderMetadatas['entries']) > 0) {
                foreach ($folderMetadatas['entries'] as $f) {
                    if ($f['.tag'] === 'folder') {
                        $listfolder[$f['id']] = $f;
                    }
                }
            }
        } catch (Exception $ex) {
            $this->lastError = $ex->getMessage();

            return false;
        }

        return $listfolder;
    }

    /**
     * Get all folder
     *
     * @return array|boolean
     */
    public function listAllFolders()
    {
        $dropbox      = $this->getAccount();
        $listfolder   = array();
        $baseFolderId = $this->params->dropboxBaseFolderId;
        try {
            $folderMetadatas = $dropbox->getMetadataWithChildren($baseFolderId, true);

            if (count($folderMetadatas['entries']) > 0) {
                foreach ($folderMetadatas['entries'] as $f) {
                    if ($f['.tag'] === 'folder') {
                        $listfolder[$f['id']] = $f;
                    }
                }
            }
        } catch (Exception $ex) {
            $this->lastError = $ex->getMessage();

            return false;
        }

        return $listfolder;
    }

    /**
     * Get path dropbox
     *
     * @param array $diff_add Different
     *
     * @return array|boolean
     */
    public function getPathById($diff_add)
    {
        $dropbox   = $this->getAccount();
        $listPaths = array();
        try {
            foreach ($diff_add as $v) {
                $content                           = $dropbox->getMetadata($v);
                $listPaths[$content['path_lower']] = array(
                    'path' => $content['path_lower'],
                    'id'   => $content['id'],
                    'name' => $content['name'],
                );
            }
        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return false;
        }

        return $listPaths;
    }

    /**
     * Method download a file
     *
     * @param string $id_file File id
     *
     * @return array
     */
    public function downloadDropbox($id_file)
    {
        $dropbox = $this->getAccount();
        //download file
        $tempfile = tempnam(sys_get_temp_dir(), 'wpfd');
        $fd       = fopen($tempfile, 'wb');
        $fMeta    = $dropbox->getFile($id_file, $fd);

        return array($tempfile, $fMeta);
    }

    /**
     * Get global config
     *
     * @return array
     */
    public function getGlobalConfig()
    {
        $allowedext_list                          = '7z,ace,bz2,dmg,gz,rar,tgz,zip,csv,doc,docx,html,key,keynote,odp,ods,odt,pages,pdf,pps,ppt,"
                                                    . "pptx,rtf,tex,txt,xls,xlsx,xml,bmp,exif,gif,ico,jpeg,jpg,png,psd,tif,tiff,aac,aif,aiff,alac,amr,au,cdda,"
                                                    . "flac,m3u,m4a,m4p,mid,mp3,mp4,mpa,ogg,pac,ra,wav,wma,3gp,asf,avi,flv,m4v,mkv,mov,mpeg,mpg,rm,swf,"
                                                    . "vob,wmv,css,img';
        $extension_list                           = 'png,jpg,pdf,ppt,doc,xls,dxf,ps,eps,xps,psd,tif,tiff,bmp,svg,pages,ai,dxf,ttf,txt,mp3,mp4';
        $defaultConfig                            = array('allowedext' => $allowedext_list);
        $defaultConfig['deletefiles']             = 0;
        $defaultConfig['catparameters']           = 1;
        $defaultConfig['defaultthemepercategory'] = 'default';
        $defaultConfig['date_format']             = 'd-m-Y';
        $defaultConfig['use_google_viewer']       = 'lightbox';
        $defaultConfig['extension_viewer']        = $extension_list;
        $defaultConfig['show_file_import']        = 0;
        $defaultConfig['uri']                     = 'download';
        $defaultConfig['ga_download_tracking']    = 0;
        $config                                   = get_option('_wpfd_global_config', $defaultConfig);
        $config                                   = array_merge($defaultConfig, $config);

        return (array) $config;
    }

    /**
     * Get all files in app folder
     *
     * @param array $filters Filters
     *
     * @return array
     */
    public function getAllFilesInAppFolder($filters)
    {
        $dropbox   = $this->getAccount();
        $listfiles = array();
        if (isset($filters['catid'])) {
            $pathFolder   = WpfdAddonHelper::getPathByDropboxId($filters['catid']);
            $fileFolder   = $dropbox->getMetadataWithChildren($pathFolder, true);

            $listFileName = array();
            if (isset($filters['q'])) {
                $fileSearch = $dropbox->searchFileNames('', $filters['q']);
                foreach ($fileSearch['matches'] as $k => $v) {
                    if ($v['metadata']['.tag'] === 'file') {
                        $listFileName[] = $v['metadata']['name'];
                    }
                }
                foreach ($fileFolder['entries'] as $k => $f) {
                    if (!empty($listFileName)) {
                        if (in_array($f['name'], $listFileName)) {
                            if ($this->checkTimeCreate($f, $filters)) {
                                $listfiles[] = $f;
                            }
                        }
                    }
                }
            } else {
                foreach ($fileFolder['entries'] as $k => $f) {
                    if ($this->checkTimeCreate($f, $filters)) {
                        $listfiles[] = $f;
                    }
                }
            }
        } else {
            if (isset($filters['q'])) {
                $fileSearch = $dropbox->searchFileNames('', $filters['q']);
                foreach ($fileSearch['matches'] as $k => $v) {
                    if ($this->checkTimeCreate($v['metadata'], $filters)) {
                        $listfiles[] = $v['metadata'];
                    }
                }
            } else {
                $allFile = $dropbox->getMetadataWithChildren('', true);
                foreach ($allFile['entries'] as $k => $v) {
                    if ($v['.tag'] === 'file') {
                        if ($this->checkTimeCreate($v, $filters)) {
                            $listfiles[] = $v;
                        }
                    }
                }
            }
        }

        $result = $this->displayFileSearch($listfiles);

        return $result;
    }

    /**
     * Check Time Create
     *
     * @param array      $file    File
     * @param null|array $filters Filters
     *
     * @return boolean
     */
    private function checkTimeCreate($file, $filters = null)
    {
        $ftime  = date('Y-m-d', strtotime($file['client_modified']));
        $result = false;
        if (isset($filters['cfrom']) && isset($filters['cto'])) {
            if (strtotime($filters['cfrom']) <= strtotime($ftime) && strtotime($ftime) <= strtotime($filters['cto'])) {
                $result = true;
            }
        } elseif (isset($filters['ufrom']) && isset($filters['uto'])) {
            if (strtotime($filters['ufrom']) <= strtotime($ftime) && strtotime($ftime) <= strtotime($filters['uto'])) {
                $result = true;
            }
        } else {
            $result = true;
        }

        return $result;
    }

    /**
     * Display files search
     *
     * @param array $listFiles List files
     *
     * @return array
     */
    private function displayFileSearch($listFiles)
    {
        $result = array();
        $config = get_option('_wpfd_global_config');
        if (!empty($config) && !empty($config['date_format'])) {
            $dateFormat = $config['date_format'];
        } else {
            $dateFormat = 'd-m-Y h:i:s';
        }

        if (!empty($listFiles)) {
            foreach ($listFiles as $f) {
                if ($f['.tag'] === 'file') {
                    $fpath               = pathinfo($f['path_display']);
                    $lpath               = pathinfo($f['path_lower']);
                    $termID              = WpfdAddonHelper::getTermIdByDropboxPath($fpath['dirname']);
                    $file                = new stdClass();
                    $file->id            = $f['id'];
                    $file->ID            = $file->id;
                    $file->lpath         = $lpath['dirname'];
                    $file->rawlpath      = $f['path_lower'];
                    $file->title         = WpfdAddonHelper::stripExt($fpath['filename']);
                    $file->post_title    = $fpath['filename'];
                    $file->ext           = $fpath['extension'];
                    $file->size          = (string) $f['size'];
                    $file->created_time  = get_date_from_gmt(date('Y-m-d H:i:s', strtotime($f['client_modified'])));
                    $file->modified_time = get_date_from_gmt(date('Y-m-d H:i:s', strtotime($f['server_modified'])));
                    $file->created       = mysql2date($dateFormat, $file->created_time);
                    $file->modified      = mysql2date($dateFormat, $file->modified_time);

                    $config = WpfdAddonHelper::getDropboxFileInfos();
                    if (!empty($config) && isset($config[$termID]) && isset($config[$termID][$f['id']])) {
                        $file->file_tags = isset($config[$termID][$f['id']]['file_tags']) ?
                            $config[$termID][$f['id']]['file_tags'] : '';
                    }
                    $result[] = $file;
                }
            }
        }

        return $result;
    }

    /**
     * Get path file
     *
     * @param string $id File id
     *
     * @return mixed
     */
    public function getPathFile($id)
    {
        $dropbox = $this->getAccount();
        $meta    = $dropbox->getMetadata($id);
        $fpath   = pathinfo($meta['path_lower']);

        return $fpath['dirname'];
    }

    /**
     * Delete all local folder and OneDrive Business folder for testing purpose
     *
     * @return void
     *
     * @throws Exception Fire message if errors
     */
    public function deleteAllFolder()
    {
        $dropbox = $this->getAllDropboxConfigs();

        $base_folder = isset($dropbox['dropboxBaseFolderId']) ? $dropbox['dropboxBaseFolderId'] : false;

        if ($base_folder) {
            try {
                $folders = $this->listChildrenFolders($base_folder);

                foreach ($folders as $folderId => $folder) {
                    $termId = WpfdAddonHelper::getTermIdByDropboxPath($folderId);
                    wp_delete_term($termId, 'wpfd-category');
                    $this->deleteDropbox($folderId);
                }
            } catch (\Exception $e) {
                $this->lastError = $e->getMessage();
            }
        }
    }
}
