<?php
/**
 * WP File Download
 *
 * @package WP File Download
 * @author  Joomunited
 * @version 1.0
 */

defined('ABSPATH') || die();

use Joomunited\WPFramework\v1_0_5\Application;
use Joomunited\WPFramework\v1_0_5\Model;

/**
 * Class WpfdAddonHelper
 */
class WpfdAddonHelper
{
    /**
     * Get all cloud config
     *
     * @return mixed
     */
    public static function getAllCloudConfigs()
    {
        $default = array(
            'googleClientId'     => '',
            'googleClientSecret' => '',
            'googleSyncTime'     => '30',
            'googleSyncMethod'   => 'sync_page_curl',
            'googleConnectedBy'  => get_current_user_id()
        );
        return get_option('_wpfdAddon_cloud_config', $default);
    }


    /**
     * Get cloud config old
     *
     * @return mixed
     */
    public static function getAllCloudConfigsOld()
    {
        return get_option('_wpfdAddon_cloud_config_old');
    }

    /**
     * Save cloud config
     *
     * @param array $data Cloud config data
     *
     * @return boolean
     */
    public static function saveCloudConfigs($data)
    {
        $result = update_option('_wpfdAddon_cloud_config', $data);

        return $result;
    }

    /**
     * Save cloud config old
     *
     * @param array $data Cloud config old data
     *
     * @return boolean
     */
    public static function saveCloudConfigsOld($data)
    {
        $result = update_option('_wpfdAddon_cloud_config_old', $data);

        return $result;
    }

    /**
     * Get data config by server name
     *
     * @param string $name Config name
     *
     * @return array|null
     */
    public static function getDataConfigBySeverName($name)
    {
        $googleDriveParams = array();
        $params = self::getAllCloudConfigs();
        if ($params && is_countable($params)) {
            foreach ($params as $key => $val) {
                if (strpos($key, 'google') !== false) {
                    $googleDriveParams[$key] = $val;
                }
            }
            $result = null;
            switch ($name) {
                case 'google':
                    $result = $googleDriveParams;
                    break;
            }

            return $result;
        }

        return array();
    }

    /**
     * Get all cloud params
     *
     * @return mixed
     */
    public static function getAllCloudParams()
    {
        return get_option('_wpfdAddon_cloud_category_params');
    }

    /**
     * Get google drive params
     *
     * @return mixed
     */
    public static function getGoogleDriveParams()
    {
        $params = self::getAllCloudParams();
        return isset($params['googledrive']) ? $params['googledrive'] : false;
    }

    /**
     * Set cloud params
     *
     * @param string $key Option key
     * @param mixed  $val Option value
     *
     * @return mixed
     */
    public static function setCloudParam($key, $val)
    {
        $params       = self::getAllCloudConfigs();
        $params[$key] = $val;
        self::saveCloudConfigs($params);
    }


    /**
     * Get google drive by google id
     *
     * @param string $googleDriveId Google Drive Id
     *
     * @return boolean
     */
    public static function getTermIdGoogleDriveByGoogleId($googleDriveId)
    {
        global $wpdb;
        $row = $wpdb->get_row($wpdb->prepare('SELECT term_id, meta_value FROM ' . $wpdb->termmeta . ' WHERE meta_key = %s AND meta_value = %s', array('wpfd_drive_id', $googleDriveId)));
        if ($row) {
            $driveType =  get_term_meta($row->term_id, 'wpfd_drive_type', true);
            if (!is_wp_error($driveType) && $driveType === 'googleDrive') {
                return isset($row->term_id)? (int) $row->term_id : false;
            }
        }

        return false;
    }

    /**
     * Get google drive id by term id
     *
     * @param integer $termId Term id
     *
     * @return boolean
     */
    public static function getGoogleDriveIdByTermId($termId)
    {
        $result = get_term_meta($termId, 'wpfd_drive_id', true);
        $type = get_term_meta($termId, 'wpfd_drive_type', true);

        if ($result && $type === 'googleDrive') {
            return $result;
        }

        return false;
    }

    /**
     * Get cat id by cloud
     *
     * @param string $cloud_id Cloud
     *
     * @return boolean
     */
    public static function getCatIdByCloudId($cloud_id)
    {
        global $wpdb;
        $row = $wpdb->get_row($wpdb->prepare('SELECT term_id, meta_value FROM ' . $wpdb->termmeta . ' WHERE meta_key = %s AND meta_value = %s', array('wpfd_drive_id', $cloud_id)));
        if ($row) {
            return isset($row->term_id)? (int) $row->term_id : false;
        }

        return false;
    }

    /**
     * Get all google drive
     *
     * @return array
     */
    public static function getAllGoogleDriveId()
    {
        global $wpdb;
        $results = $wpdb->get_results('SELECT meta_value from ' . $wpdb->termmeta . ' WHERE meta_key = \'wpfd_drive_id\' AND meta_id IN (SELECT meta_id FROM ' . $wpdb->termmeta . ' WHERE meta_key = \'wpfd_drive_type\' AND meta_value = \'googleDrive\')', ARRAY_A);
        return array_map(function ($result) {
            return array('idCloud' => $result['meta_value']);
        }, $results);
    }

    /**
     * Curl sync Interval
     *
     * @return float|integer
     */
    public static function curSyncInterval()
    {
        //get last_log param
        $config = self::getAllCloudConfigs();
        if (isset($config['last_log']) && !empty($config['last_log'])) {
            $last_log = $config['last_log'];
            $last_sync = (int)strtotime($last_log);
        } else {
            $last_sync = 0;
        }
        $time_new = (int)strtotime(date('Y-m-d H:i:s'));
        $timeInterval = $time_new - $last_sync;
        $curtime = $timeInterval / 60;
        return $curtime;
    }

    /**
     * Get ext
     *
     * @param string $file File name
     *
     * @return boolean|string
     */
    public static function getExt($file)
    {
        $dot = strrpos($file, '.') + 1;

        return substr($file, $dot);
    }

    /**
     * Strips the last extension off of a file name
     *
     * @param string $file The file name
     *
     * @return string The file name without the extension
     *
     * @since 11.1
     */
    public static function stripExt($file)
    {
        return preg_replace('#\.[^.]*$#', '', $file);
    }

    /**
     * Write log only in debug mode
     *
     * @param mixed $log Log data
     *
     * @return void
     */
    public static function writeLog($log)
    {
        if (defined('WP_DEBUG') && WP_DEBUG) {
            // phpcs:disable WordPress.PHP.DevelopmentFunctions.error_log_print_r, WordPress.PHP.DevelopmentFunctions.error_log_error_log -- This is login function
            if (is_array($log) || is_object($log)) {
                error_log(print_r($log, true));
            } else {
                error_log($log);
            }
            // phpcs:enable
        }
    }

    /*----------- Dropbox -----------------*/
    /**
     * Get all dropbox config
     *
     * @return mixed
     */
    public static function getAllDropboxConfigs()
    {
        $default = array(
            'dropboxKey'          => '',
            'dropboxSecret'       => '',
            'dropboxBaseFolderId' => '',
            'dropboxSyncTime'     => '30',
            'last_log'            => '0',
            'dropboxSyncMethod'   => 'sync_page_curl',
            'dropboxConnectedBy'  => get_current_user_id()
        );
        return get_option('_wpfdAddon_dropbox_config', $default);
    }

    /**
     * Set dropbox param
     *
     * @param string $key Param key
     * @param mixed  $val Param Value
     *
     * @return void
     */
    public static function setDropboxParam($key, $val)
    {
        $params = self::getAllDropboxConfigs();
        $params[$key] = $val;
        self::saveDropboxConfigs($params);
    }
    /**
     * Save dropbox config
     *
     * @param array $data Config data
     *
     * @return boolean
     */
    public static function saveDropboxConfigs($data)
    {
        $result = update_option('_wpfdAddon_dropbox_config', $data);

        return $result;
    }

    /**
     * Get data config by dropbox
     *
     * @param string $name Data name
     *
     * @return array|null
     */
    public static function getDataConfigByDropbox($name)
    {
        $DropboxParams = array();
        if (self::getAllDropboxConfigs()) {
            foreach (self::getAllDropboxConfigs() as $key => $val) {
                if (strpos($key, 'dropbox') !== false) {
                    $DropboxParams[$key] = $val;
                }
            }
            $result = null;
            switch ($name) {
                case 'dropbox':
                    $result = $DropboxParams;
                    break;
            }

            return $result;
        }

        return null;
    }


    /**
     * Get id by termID
     *
     * @param integer $termId Term id
     *
     * @return boolean|string
     */
    public static function getDropboxIdByTermId($termId)
    {
        $result = get_term_meta($termId, 'wpfd_drive_id', true);
        $type = get_term_meta($termId, 'wpfd_drive_type', true);

        if ($result && $type === 'dropbox') {
            return $result;
        }

        return false;
    }
    /**
     * Get path by termID
     *
     * @param integer $termId Term id
     *
     * @return boolean|string
     */
    public static function getDropboxPathByTermId($termId)
    {
        return get_term_meta($termId, 'wpfd_drive_path', true);
    }

    /**
     * Get term id by Path
     *
     * @param string $path Dropbox path
     *
     * @return boolean|integer|string
     */
    public static function getTermIdByDropboxPath($path)
    {
        global $wpdb;
        $row = $wpdb->get_row($wpdb->prepare('SELECT term_id, meta_value FROM ' . $wpdb->termmeta . ' WHERE meta_key = %s AND meta_value = %s', array('wpfd_drive_path', $path)));

        if ($row) {
            $driveType =  get_term_meta($row->term_id, 'wpfd_drive_type', true);
            if (!is_wp_error($driveType) && $driveType === 'dropbox') {
                return isset($row->term_id)? (int) $row->term_id : false;
            }
        }

        return false;
    }

    /**
     * Get path by id
     *
     * @param string $id Dropbox item id
     *
     * @return string|boolean
     */
    public static function getPathByDropboxId($id)
    {
        global $wpdb;
        $row = $wpdb->get_row($wpdb->prepare('SELECT term_id, meta_value FROM ' . $wpdb->termmeta . ' WHERE meta_key = %s AND meta_value = %s', array('wpfd_drive_id', $id)));

        if ($row) {
            $termId = isset($row->term_id) ? $row->term_id : false;
            if ($termId) {
                return get_term_meta($termId, 'wpfd_drive_path', true);
            }
        }
        return false;
    }

    /**
     * Set dropbox file info
     *
     * @param array $params Dropbox file info
     *
     * @return boolean
     */
    public static function setDropboxFileInfos($params)
    {
        $result = update_option('_wpfdAddon_dropbox_fileInfo', $params, false);

        return $result;
    }

    /**
     * Get Dropbox file info
     *
     * @return mixed
     */
    public static function getDropboxFileInfos()
    {
        return get_option('_wpfdAddon_dropbox_fileInfo');
    }

    /**
     * Curl sync interval dropbox
     *
     * @return float|integer
     */
    public static function curSyncIntervalDropbox()
    {
        // Get last_log param
        $config = self::getAllDropboxConfigs();
        if (isset($config['last_log']) && !empty($config['last_log'])) {
            $last_log = $config['last_log'];
            $last_sync = (int)strtotime($last_log);
        } else {
            $last_sync = 0;
        }
        $time_new = (int)strtotime(date('Y-m-d H:i:s'));
        $timeInterval = $time_new - $last_sync;
        $curtime = $timeInterval / 60;

        return $curtime;
    }
    /*----------- OneDrive Business --------*/
    /**
     * Get all OneDrive Business config
     *
     * @return array
     */
    public static function getAllOneDriveBusinessConfigs()
    {
        $default = array(
            'onedriveBusinessKey'         => '',
            'onedriveBusinessSecret'      => '',
            'onedriveBusinessSyncTime'    => '30',
            'onedriveBusinessSyncMethod'  => 'sync_page_curl',
            'onedriveBusinessConnectedBy' => get_current_user_id(),
            'state'                       => array()
        );

        return get_option('_wpfdAddon_onedrive_business_config', $default);
    }
    /**
     * Save OneDrive business config
     *
     * @param array $data Onedrive config data
     *
     * @return boolean
     */
    public static function saveOneDriveBusinessConfigs($data)
    {
        $config = self::getAllOneDriveBusinessConfigs();
        $config['onedriveBusinessKey'] = $data['onedriveBusinessKey'];
        $config['onedriveBusinessSecret'] = $data['onedriveBusinessSecret'];
        $config['onedriveBusinessSyncTime'] = $data['onedriveBusinessSyncTime'];
        $config['onedriveBusinessSyncMethod'] = $data['onedriveBusinessSyncMethod'];
        if (isset($data['onedriveBaseFolder'])) {
            $config['onedriveBaseFolder'] = $data['onedriveBaseFolder'];
        }
        return update_option('_wpfdAddon_onedrive_business_config', $config);
    }

    /**
     * Get onedrive by term id
     *
     * @param integer $termId Term id
     *
     * @return boolean
     */
    public static function getOneDriveBusinessIdByTermId($termId)
    {
        $result = get_term_meta($termId, 'wpfd_drive_id', true);
        $type = get_term_meta($termId, 'wpfd_drive_type', true);

        if ($result && $type === 'onedrive_business') {
            return $result;
        }

        return false;
    }
    /**
     * Get onedrive business file info
     *
     * @return mixed
     */
    public static function getOneDriveBusinessFileInfos()
    {
        return get_option('_wpfdAddon_onedrive_business_fileInfo');
    }
    /**
     * Set onedrive business file info
     *
     * @param array $params Onedrive file info
     *
     * @return boolean
     */
    public static function setOneDriveBusinessFileInfos($params)
    {
        return update_option('_wpfdAddon_onedrive_business_fileInfo', $params);
    }
    /**
     * Set onedrive param
     *
     * @param string $key Option key to save
     * @param mixed  $val Option value to save
     *
     * @return void
     */
    public static function setOneDriveBusinessParam($key, $val)
    {
        $params       = self::getAllOneDriveBusinessConfigs();
        $params[$key] = $val;
        self::saveOneDriveBusinessConfigs($params);
    }

    /**
     * Get term id onedrive business by onedrive
     *
     * @param string $oneDriveId Onedrive item id
     *
     * @return boolean|integer
     */
    public static function getTermIdOneDriveBusinessByOneDriveId($oneDriveId)
    {
        global $wpdb;
        $row = $wpdb->get_row($wpdb->prepare('SELECT term_id, meta_value FROM ' . $wpdb->termmeta . ' WHERE meta_key = %s AND meta_value = %s', array('wpfd_drive_id', $oneDriveId)));
        if ($row) {
            $driveType =  get_term_meta($row->term_id, 'wpfd_drive_type', true);
            if (!is_wp_error($driveType) && $driveType === 'onedrive_business') {
                return isset($row->term_id)? (int) $row->term_id : false;
            }
        }

        return false;
    }
    /*----------- OneDrive -----------------*/

    /**
     * Get all onedrive config
     *
     * @return mixed
     */
    public static function getAllOneDriveConfigs()
    {
        $default = array(
            'onedriveKey'         => '',
            'onedriveSecret'      => '',
            'onedriveSyncTime'    => '30',
            'onedriveSyncMethod'  => 'sync_page_curl',
            'onedriveConnectedBy' => get_current_user_id(),
            'onedriveState'       => array(),
        );
        return get_option('_wpfdAddon_onedrive_config', $default);
    }

    /**
     * Get alll onedrive config old
     *
     * @return mixed
     */
    public static function getAllOneDriveConfigsOld()
    {
        return get_option('_wpfdAddon_onedrive_config_old');
    }


    /**
     * Save onedrive config
     *
     * @param array $data Onedrive config data
     *
     * @return boolean
     */
    public static function saveOneDriveConfigs($data)
    {
        $config                               = self::getAllOneDriveConfigs();

        $config['onedriveKey']        = $data['onedriveKey'];
        $config['onedriveSecret']     = $data['onedriveSecret'];
        $config['onedriveSyncTime']   = $data['onedriveSyncTime'];
        $config['onedriveSyncMethod'] = $data['onedriveSyncMethod'];
        $config['last_log'] = $data['last_log'];
        $config['onedriveConnectedBy'] = $data['onedriveConnectedBy'];

        if (isset($data['onedriveBaseFolder'])) {
            $config['onedriveBaseFolder']  = $data['onedriveBaseFolder'];
        }
        if (isset($data['onedriveConnected'])) {
            $config['onedriveConnected']  = $data['onedriveConnected'];
        }
        if (isset($data['onedriveState'])) {
            $config['onedriveState']  = $data['onedriveState'];
        }

        $result = update_option('_wpfdAddon_onedrive_config', $config);

        return $result;
    }

    /**
     * Save onedrive config old
     *
     * @param array $data Onedrive config data old
     *
     * @return boolean
     */
    public static function saveOneDriveConfigsOld($data)
    {
        $result = update_option('_wpfdAddon_onedrive_config_old', $data);

        return $result;
    }

    /**
     * Get all onedrive params
     *
     * @return mixed
     */
    public static function getAllOneDriveParams()
    {
        return get_option('_wpfdAddon_onedrive_category_params');
    }

    /**
     * Get onedrive param
     *
     * @return mixed
     */
    public static function getOneDriveParams()
    {
        $params = self::getAllOneDriveParams();
        return isset($params['onedrive']) ? $params['onedrive'] : false;
    }

    /**
     * Get data config by onedrive
     *
     * @param string $name Get data by key name
     *
     * @return array|null
     */
    public static function getDataConfigByOneDrive($name)
    {
        $OneDriveParams = array();
        if (self::getAllOneDriveConfigs()) {
            foreach (self::getAllOneDriveConfigs() as $key => $val) {
                if (strpos($key, 'onedrive') !== false) {
                    $OneDriveParams[$key] = $val;
                }
            }
            $result = null;
            switch ($name) {
                case 'onedrive':
                    $result = $OneDriveParams;
                    break;
            }

            return $result;
        }

        return null;
    }

    /**
     * Get onedrive by term id
     *
     * @param integer $termId Term id
     *
     * @return boolean
     */
    public static function getOneDriveIdByTermId($termId)
    {
        $result = get_term_meta($termId, 'wpfd_drive_id', true);
        $type = get_term_meta($termId, 'wpfd_drive_type', true);

        if ($result && $type === 'onedrive') {
            $result = self::replaceIdOneDrive($result, false);

            return $result;
        }

        return false;
    }

    /**
     * Get term id onedrive by onedrive
     *
     * @param string $oneDriveId Onedrive item id
     *
     * @return boolean
     */
    public static function getTermIdOneDriveByOneDriveId($oneDriveId)
    {
//        $returnData = false;
        global $wpdb;
        $oneDriveId = self::replaceIdOneDrive($oneDriveId);

        $row = $wpdb->get_row($wpdb->prepare('SELECT term_id, meta_value FROM ' . $wpdb->termmeta . ' WHERE meta_key = %s AND meta_value = %s', array('wpfd_drive_id', $oneDriveId)));
        if ($row) {
            $driveType =  get_term_meta($row->term_id, 'wpfd_drive_type', true);
            if (!is_wp_error($driveType) && $driveType === 'onedrive') {
                return isset($row->term_id)? (int) $row->term_id : false;
            }
        }

        return false;
//        $onedriveParams = self::getOneDriveParams();
//        if ($onedriveParams) {
//            foreach ($onedriveParams as $key => $val) {
//                if ($val['oneDriveId'] === $oneDriveId) {
//                    $returnData = $val['termId'];
//                }
//            }
//        }
//
//        return $returnData;
    }

    /**
     * Get all onedrive id
     *
     * @return array
     */
    public static function getAllOneDriveId()
    {
        $returnData     = array();
        $onedriveParams = self::getOneDriveParams();
        if ($onedriveParams) {
            foreach ($onedriveParams as $key => $val) {
                $returnData[] = $val['oneDriveId'];
            }
        }

        return $returnData;
    }

    /**
     * Set onedrive param
     *
     * @param string $key Option key to save
     * @param mixed  $val Option value to save
     *
     * @return void
     */
    public static function setOneDriveParam($key, $val)
    {
        $params       = self::getAllOneDriveConfigs();
        $params[$key] = $val;
        self::saveOneDriveConfigs($params);
    }

    /**
     * Set onedrive file info
     *
     * @param array $params Onedrive file info
     *
     * @return boolean
     */
    public static function setOneDriveFileInfos($params)
    {
        $result = update_option('_wpfdAddon_onedrive_fileInfo', $params);

        return $result;
    }

    /**
     * Get onedrive file info
     *
     * @return mixed
     */
    public static function getOneDriveFileInfos()
    {
        return get_option('_wpfdAddon_onedrive_fileInfo');
    }

    /**
     * Get Onedrive term id by file id
     *
     * @param string $fileId File Id
     *
     * @return boolean|string
     */
    public static function getOneDriveTermIdByCloudId($fileId)
    {
        $fileInfos = self::getOneDriveFileInfos();
        if (!empty($fileInfos)) {
            foreach ($fileInfos as $termId => $fileInfo) {
                foreach ($fileInfo as $idFile => $fileParam) {
                    if ((string) $fileId === (string) $idFile) {
                        return $termId;
                    }
                }
            }
        }

        return false;
    }

    /**
     * Replace replace id special characters
     *
     * @param string  $id         Item id
     * @param boolean $rplSpecial Replace from special to -
     *
     * @return string
     */
    public static function replaceIdOneDrive($id, $rplSpecial = true)
    {
        if ($rplSpecial) {
            return str_replace('!', '-', $id);
        } else {
            return str_replace('-', '!', $id);
        }
    }

    /**
     * Curl sync interval onedrive
     *
     * @return float|integer
     */
    public static function curSyncIntervalOneDrive()
    {
        //get last_log param
        $config = self::getAllOneDriveConfigs();
        if (isset($config['last_log']) && !empty($config['last_log'])) {
            $last_log = $config['last_log'];
            $last_sync = (int)strtotime($last_log);
        } else {
            $last_sync = 0;
        }

        $time_new = (int)strtotime(date('Y-m-d H:i:s'));
        $timeInterval = $time_new - $last_sync;
        $curtime = $timeInterval / 60;

        return $curtime;
    }

    /**
     * Save category param
     *
     * @param integer $id         Term id
     * @param string  $type_cloud Cloud type
     *
     * @return boolean
     */
    public static function updateDefaultCategoryParams($id, $type_cloud)
    {
        Application::getInstance('Wpfd');
        $wpfd_config   = Model::getInstance('config');
        $defult_params = $wpfd_config->getThemeParams('default');

        Application::getInstance('WpfdAddon');
        $google_config   = self::getAllCloudConfigs();
        $dropbox_config  = self::getAllDropboxConfigs();
        $onedrive_config = self::getAllOneDriveConfigs();
        $onedrive_business_config = self::getAllOneDriveBusinessConfigs();

        switch ($type_cloud) {
            case 'google':
                $category_own = $google_config['googleConnectedBy'];
                break;
            case 'dropbox':
                $category_own = $dropbox_config['dropboxConnectedBy'];
                break;
            case 'onedrive':
                $category_own = $onedrive_config['onedriveConnectedBy'];
                break;
            case 'onedrive_business':
                $category_own = $onedrive_business_config['onedriveBusinessConnectedBy'];
                break;
            default:
                $category_own = get_current_user_id();
        }

        $params = array(
            'theme'            => 'default',
            'canview'          => 0,
            'social'           => 0,
            'category_own'     => $category_own,
            'category_own_old' => $category_own
        );
        $params = array_merge($params, $defult_params);

        $datas = json_encode($params);
        $updated = wp_update_term($id, 'wpfd-category', array('description' => $datas));
        if (is_wp_error($updated)) {
            return false;
        }

        return true;
    }

    /**
     * Delete category in user meta
     *
     * @param integer $cat_id            Term id
     * @param array   $list_category_own Own categories
     *
     * @return void
     */
    public static function delCatInUserMeta($cat_id, $list_category_own)
    {
        $user_id = get_current_user_id();
        if (!in_array($user_id, $list_category_own)) {
            $list_category_own[] = $user_id;
        }

        foreach ($list_category_own as $key => $own) {
            $user_categories = (array)get_user_meta($own, 'wpfd_user_categories', true);
            if (is_array($user_categories)) {
                foreach ($user_categories as $uc_key => $uc_cat_id) {
                    if ((int)$cat_id === (int)$uc_cat_id) {
                        unset($user_categories[$uc_key]);
                    }
                }
                update_user_meta($own, 'wpfd_user_categories', $user_categories);
            }
        }
    }
}
